"""
 * ***** BEGIN LICENSE BLOCK *****
 * Portions Copyright (c) 1996-2004 RealNetworks, Inc. All Rights Reserved.
 * 
 * The contents of this file, and the files included with this file,
 * are subject to the current version of the RealNetworks Public
 * Source License (the "RPSL") available at
 * http://www.helixcommunity.org/content/rpsl unless you have licensed
 * the file under the current version of the RealNetworks Community
 * Source License (the "RCSL") available at
 * http://www.helixcommunity.org/content/rcsl, in which case the RCSL
 * will apply. You may also obtain the license terms directly from
 * RealNetworks.  You may not use this file except in compliance with
 * the RPSL or, if you have a valid RCSL with RealNetworks applicable
 * to this file, the RCSL.  Please see the applicable RPSL or RCSL for
 * the rights, obligations and limitations governing use of the
 * contents of the file.
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL") in which case the provisions of the GPL are applicable
 * instead of those above. If you wish to allow use of your version of
 * this file only under the terms of the GPL, and not to allow others
 * to use your version of this file under the terms of either the RPSL
 * or RCSL, indicate your decision by deleting the provisions above
 * and replace them with the notice and other provisions required by
 * the GPL. If you do not delete the provisions above, a recipient may
 * use your version of this file under the terms of any one of the
 * RPSL, the RCSL or the GPL.
 * 
 * This file is part of the Helix DNA Technology. RealNetworks is the
 * developer of the Original Code and owns the copyrights in the
 * portions it created.
 * 
 * This file, and the files included with this file, is distributed
 * and made available on an 'AS IS' basis, WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, AND REALNETWORKS HEREBY DISCLAIMS
 * ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 * 
 * Technology Compatibility Kit Test Suite(s) Location:
 *    http://www.helixcommunity.org/content/tck
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK *****
"""

import os
import sys
import gtk
import gobject
import string
import pango

from gettext import gettext as _

from sugar.graphics.toolbutton import ToolButton
from sugar.graphics.toggletoolbutton import ToggleToolButton
from sugar.graphics.icon import Icon
from sugar.activity import activity

os.environ["HELIX_LIBS"] = activity.get_bundle_path() + '/helix'
helix_libs = os.environ.get("HELIX_LIBS")
sys.path.append(helix_libs)
import hxplay

MAX_WIDTH = 1200
MAX_HEIGHT = 764

def aspectSafeResize(player, width, height):
    oldWidth, oldHeight = player.getSize()
    if (width >= height):
        newWidth = width
        newHeight = (oldHeight * newWidth) / oldWidth
        if (newHeight > MAX_HEIGHT):
            newHeight = height
            newWidth = (oldWidth * newHeight) / oldHeight
    else:
        newHeight = height
        newWidth = (oldWidth * newHeight) / oldHeight
        if (newWidth > MAX_WIDTH):
            newWidth = width
            newHeight = (oldHeight * newWidth) / oldWidth
    player.setSize(int(newWidth),int(newHeight))


def positionInCenter(window, player):
    playerWidth, playerHeight = player.getSize()
    windowWidth, windowHeight = window.size_request()

    x = (windowWidth / 2) - (playerWidth / 2)
    y = (windowHeight / 2) - (playerHeight / 2)

    player.setPosition(int(x), int(y))

class MediaPlayerActivity(activity.Activity):
  
    def idle_function(self):
        try:
            hxplay.doevent()
        except Exception, e:
            print e
        return True

    def previous_clicked(self, widget):
        if (self.player != None) and (self.playing == 1):
            self.player.previous()

    def next_clicked(self, widget):
        #group index starts at 0, so we need +1
        if self.player and self.playing and \
        (self.player.currentGroup() + 1 < self.player.numGroups()):
            self.player.next()
        elif (self.player.currentGroup() + 1 == self.player.numGroups()) and self.repeat:
            self.player.goToGroup(0)

    def repeat_clicked(self, widget):
        if (self.player != None) and (self.playing == 1):
            self.toggle_repeat()

    def toggle_repeat(self):
        if (self.repeat == 0):
            self.repeat = 1
            self.repeat_button.set_icon("norepeat")
            self.repeat_button.set_tooltip(_("No Repeat"))
        else:
            self.repeat = 0
            self.repeat_button.set_icon("repeat")
            self.repeat_button.set_tooltip(_("Repeat"))

    def play_clicked(self, widget):
        self.play()

    def stop_clicked(self, widget):
        self.stop()

    def play(self, force=None):
        if (self.player != None):                       
            if self.playing or force:
                    self.playing=0
                    self.play_button.set_icon("stock-play")
                    self.play_button.set_tooltip(_("Play"))
                    gobject.source_remove(self.idlefunction)
                    self.player.pause()
            else:             
                if self.stopped:
                    self.openfile(self.file)
                    self.paused = 0
                else:
                    self.paused = 0
                    self.playing = 1
                    self.idlefunction = gobject.idle_add(self.idle_function)
                    self.stopped = 0
                    self.play_button.set_icon("stock-pause")
                    self.play_button.set_tooltip(_('Pause'))
                    self.player.start()

        elif self.file:
            self.openfile(self.file)
    
    def handle_uri_timer(self):
        if (self.file) and (not self.player):
            self.openfile(self.file)
        return False

    def stop(self):
        if (self.player != None):
            if self.playing == 1:
                self.play_button.set_icon("stock-play")
                self.play_button.set_tooltip(_("Play"))
                self.fit_to_screen.set_icon("zoom-native")
                self.fit_to_screen.set_tooltip(_("Native Size"))
            self.progressbar_blank()
            if self.idlefunction:
               gobject.source_remove(self.idlefunction)
            self.playing = 0
            self.blank()
            self.stopped = 1
            self.paused = 0
            self.player.stop()
    
    def progressbar_blank(self):
        if (self.player != None):
            self.toolbar_label.set_markup("<span foreground=\"white\" size=\"x-large\">00:00:00/" + self.total_time + "</span>")
            self.adjustment.value = 0
        else:
            self.toolbar_label.set_markup("<span foreground=\"white\" size=\"x-large\">00:00:00/00:00:00</span>")
            self.adjustment.value = 0

    def blank(self):
        width, height = self.player.getSize()
        self.player.setSize(1, 1)
        self.subwin.hide()
        self.subwin.show()
        self.player.setSize(width, height)

    def browse_files(self, widget):
        if self.playing:
            self.play()
       
        dialog = gtk.FileChooserDialog("Open",None,gtk.FILE_CHOOSER_ACTION_OPEN,
                                      (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, 
                                       gtk.STOCK_OPEN, gtk.RESPONSE_OK))

        if self.lastDir:
            dialog.set_current_folder(self.lastDir)
        
        for filter in self.file_filters:
            dialog.add_filter(filter)

        dialog.set_default_response(gtk.RESPONSE_OK)

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            self.file = dialog.get_uri()
            self.lastDir = dialog.get_current_folder()
            self.file_entry.prepend_text(self.file)
            self.file_entry.set_active(0)

        dialog.destroy()


    def openfiledialog(self, widget):
        if self.playing:
            self.play()

        dialog = gtk.Dialog("Open A Location",
                            None,
                            gtk.DIALOG_MODAL,
                            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                            gtk.STOCK_OPEN, gtk.RESPONSE_OK))

        dialog.set_default_response(gtk.RESPONSE_OK)
        hbox = gtk.HBox(False, 12)
        hbox.set_border_width(12)
        dialog.vbox.pack_start(hbox, True, True, 0)
        hbox.show()

        label = gtk.Label('Location:')
        hbox.pack_start(label, False, False, 0)
        label.show()

        self.file_entry = gtk.ComboBoxEntry(self.file_list, 0)
        hbox.pack_start(self.file_entry)
        self.file_entry.show()
        
        browse_button = gtk.Button("Browse")
        browse_button.connect("clicked", self.browse_files)
        hbox.pack_start(browse_button)
        browse_button.show()
	
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            if(self.playing):
                self.stop()
            self.file = self.file_entry.child.get_text()
            if self.file_entry.get_active() == -1 and self.file:
                self.file_entry.prepend_text(self.file)
            self.openfile(self.file)
        else:
            if not self.stopped:
                self.play()
        dialog.destroy()
        

    def openfile(self, file = None):
        if self.file:
            print self.file
            xid = self.subwin.window.xid
            xdisplayname = self.subwin.get_screen().make_display_name()
            site = (xdisplayname,xid)
            callbacks = [
                    (hxplay.ON_PRESENTATION_OPENED, self.on_presentation_opened, None),
                    (hxplay.ON_ERROR,               self.on_error,               None),
                    (hxplay.ON_PRESENTATION_CLOSED, self.on_presentation_closed, None),
                    (hxplay.ON_POS_LENGTH,          self.on_pos_length,          None),
                    (hxplay.ON_BUFFERING,           self.on_buffering,           None),
                    (hxplay.ON_PAUSE,               self.on_pause,               None),
                    (hxplay.ON_CHANGING_SIZE,       self.on_changing_size,       None),
                    (hxplay.ON_STOP,                self.on_stop,                None)
                    ]
            
            if (self.player == None):
                self.player = hxplay.open(file,
                       callbacks=callbacks, site=site)
            else:
                if not self.stopped:
                    self.player.stop()
                self.player.open(file)
            
            self.stopped = 0 
            self.playing = 0
            self.play()

    def on_pause(self, player, data, data1, data2):
        self.paused = 1

    def generateLabel(self):

        title = self.player.stats("Title")
        author = self.player.stats("Author")

        if not title:
            title = "Unknown"

        if not author:
            author = "Unknown"

        self.label.set_markup("<span foreground=\"white\" size=\"x-large\">" + \
                              "Title: " + title + "  |  " + \
                              "Author: " + author + "</span>")
    
    def on_buffering(self, player, data, data1, data2):
        ulArgs, bufferPercent = data2
        if (bufferPercent == 100):
            self.bufferText = ""
            self.buffering = 0
            self.generateLabel()
        else:
            self.bufferText = "Buffering (%d%%)" % (bufferPercent)
            self.buffering = 1
            self.toolbar_label.set_markup("<span foreground=\"white\" size=\"x-large\">" + self.bufferText + "</span>")

    def error_dialog(self, msg, userstring = None):
        dialog = gtk.Dialog("Error",
                            None,
                            gtk.DIALOG_MODAL,
                            (gtk.STOCK_OK, gtk.RESPONSE_OK))

        dialog.set_default_response(gtk.RESPONSE_OK)
        hbox = gtk.HBox(False, 12)
        hbox.set_border_width(12)
        dialog.vbox.pack_start(hbox, True, True, 0)
        hbox.show()

        if userstring != None:
            label = gtk.Label(msg + "\n(" + userstring + ")")
        else:
            label = gtk.Label(msg)

        hbox.pack_start(label, False, False, 0)
        label.show()

        response = dialog.run()
        dialog.destroy()

    def on_error(self, player, one, two, three):
        severity, hxcode, usercode, msg, userstring, moreinfo = three
        self.error_dialog(msg, userstring)

    def on_changing_size(self, player, data, data1, data2):
        #oldx, oldy, newx, newy = data2
        # XXXgfw This is just a hack for now. We can not call aspectSafeResize
        # or positionInCenter from here because it will result in a recursive
        # callstack until the stack blows up. So, we check in ON_POS_LENGTH and
        # do it there, yuk. Once we have buttons for 1x, wide screen and full
        # screen we won't need this ugliness and it can be yanked out.
        if self.playing:
            self.needRepos = 1
        
    def on_pos_length(self,player,cb,args,(pos,length)):
        if self.playing:
            if self.int_time != int(self.player.time()):
                self.int_time = int(self.player.time())
                self.time_elapsed = ("%02d" % (self.int_time/3600)) + \
                    ":" + ("%02d" % ((self.int_time/60)%60)) + \
                    ":" + ("%02d" % (self.int_time%60))
                #if time/dimensions failed to set on init
                if not self.int_length:
                    self.set_time()
                if not self.original_height:
                    self.original_width, self.original_height = self.player.getSize()
                self.toolbar_label.set_markup("<span foreground=\"white\" size=\"x-large\">" + self.time_elapsed + "/" + self.total_time + "</span>")
                self.adjustment.value = self.int_time
        #only update the progress bar when its not being dragged
        if (self.needRepos):
            positionInCenter(self.subwin, self.player)
            self.needRepos=0
        if (not self.sizeupdated and self.player.hasSite() and not self.fit):
            self.sizeupdated = 1
            aspectSafeResize(self.player, MAX_WIDTH, MAX_HEIGHT)
            positionInCenter(self.subwin, self.player)
    
    def on_presentation_closed(self, player, one, two, three):
        if self.fullscreen:
            self.fullscreen = 0
            self.player.exitFullScreen()
        self.stop()

    def on_presentation_opened(self, player, one, two, three):
        self.blank()
        self.currentPos = 0
        self.set_time()
        self.original_width, self.original_height = self.player.getSize()
        self.toolbar_label.set_markup("<span foreground=\"white\" size=\"x-large\">00:00:00/" + self.total_time + "</span>")

        self.adjustment.value = 0
        self.adjustment.lower = 0
       
        self.sizeupdated = 0

    def on_stop(self, player, one, two, three):
        if (self.repeat == 1 and self.stopped == 0):
            repeat_timer = gobject.timeout_add(10, self.repeat_timer)

    def repeat_timer(self):
        self.play()
        return False

    def set_time(self):
        self.int_length = int(self.player.length())
        self.total_time =   ("%02d" % (self.int_length/3600)) + \
            ":" + ("%02d" % ((self.int_length/60)%60)) + \
            ":" + ("%02d" % (self.int_length%60))
        self.adjustment.upper = self.int_length
        

    def fullscreen_clicked(self, widget):
        if self.playing:
            self.player.enterFullScreen()

    def fit_screen_clicked(self, widget):
        if self.playing:
            if self.fit == 0: #widget.get_active():
                aspectSafeResize(self.player, self.original_width, self.original_height)
                positionInCenter(self.subwin, self.player)
                self.fit_to_screen.set_icon("zoom-best-fit")
                self.fit_to_screen.set_tooltip(_("Fit To Screen"))
                self.fit = 1
            else:
                aspectSafeResize(self.player, MAX_WIDTH, MAX_HEIGHT)
                positionInCenter(self.subwin, self.player)
                self.fit_to_screen.set_icon("zoom-native")
                self.fit_to_screen.set_tooltip(_("Native Size"))
                self.fit = 0
        else:
            widget.set_active(False)

    def buttondown(self, widget, event, data=None):
        if self.player != None:
	    if(self.playing):
            	self.play(0)
            	self.wasPlaying = 1

    def buttonup(self, widget, event, data=None):
        if self.player != None:
            self.player.seek(self.adjustment.value)
	    if self.wasPlaying:
	    	self.play(0)
	
	    self.wasPlaying = 0
    
    def barseek(self, widget, value=None, data=None):
        if self.player != None:
            self.int_time = int(self.adjustment.value)
            self.time_elapsed = ("%02d" % (self.int_time/3600)) + \
                ":" + ("%02d" % ((self.int_time/60)%60)) + \
                ":" + ("%02d" % (self.int_time%60))
    
        self.toolbar_label.set_markup("<span foreground=\"white\" size=\"x-large\">" + self.time_elapsed + "/" + self.total_time + "</span>")

    def __init__(self, handle):
        
        # All of our flags. Pretty important stuff as these tell us the 
        # state of our player.
        self.wasPlaying = 0
        self.fit = 0
        self.currentPos = 0
        self.run = 1
        self.playing = 0
        self.sizeupdated = 1
        self.stopped = 1
        self.player = None
        self.file = None
        self.total_time = "00:00:00"
        self.time_elapsed = "00:00:00"
        self.int_time = 0
        self.int_length = 0
        self.needRepos = 0
        self.original_width = 0
        self.original_height = 0
        self.fullscreen = 0
        self.needOriginals = 0
        self.paused = 0
        self.buffering = 0
        self.bufferText = ""
        self.repeat = 0
        self.lastDir = None
        
        self.file_list = gtk.ListStore(gobject.TYPE_STRING)

        # List of Filters(Dynamically getting these from pykit would be much better)
        self.file_filters = []
        filter = gtk.FileFilter()
        filter.add_pattern("*.mp3")
        filter.add_pattern("*.ogg")
        filter.add_pattern("*.mov")
        filter.add_pattern("*.ram")
        filter.add_pattern("*.rm")
        filter.add_pattern("*.rv")
        filter.add_pattern("*.mpg")
        filter.add_pattern("*.avi")
        filter.add_pattern("*.ogm")
        filter.add_pattern("*.mp4")
        filter.add_pattern("*.3gp")
        filter.add_pattern("*.smil")
        filter.add_pattern("*.sml")
        filter.add_pattern("*.rt")
        filter.set_name("Media Files (*.mp3, *.ogg, *.mov, *.ram, *.rm, *.rv, *.mpg, *.avi, *.ogm, *.mp4, *.3gp, *.smil, *.sml, *.rt)")
        self.file_filters.append(filter)
        filter = gtk.FileFilter()
        filter.add_pattern("*")
        filter.set_name("All Files (*.*)")
        self.file_filters.append(filter)


        activity.Activity.__init__ (self, handle)
        self.set_title("Media Player")

        toolbox = activity.ActivityToolbox(self)

        self.toolbar = gtk.Toolbar()

        #this little diddy adds our icons folder to the default theme
        self.icontheme = gtk.icon_theme_get_default()
        self.icontheme.append_search_path(activity.get_bundle_path() + '/icons')

        self.previous_button = ToolButton('stock-previous')
        self.previous_button.connect("clicked", self.previous_clicked)
        self.previous_button.set_tooltip(_('Previous'))
        self.toolbar.insert(self.previous_button, -1)
        self.previous_button.show()
        
        #Play/pause buttons.
        self.play_button = ToolButton('stock-play')
        self.play_button.connect("clicked", self.play_clicked)
        self.play_button.set_tooltip(_('Play'))
        self.toolbar.insert(self.play_button, -1)
        self.play_button.show()
     
        self.stop_button = ToolButton('stock-stop')
        self.stop_button.connect("clicked", self.stop_clicked)
        self.stop_button.set_tooltip(_('Stop'))
        self.toolbar.insert(self.stop_button, -1)
        self.stop_button.show()
        
        self.next_button = ToolButton('stock-next')
        self.next_button.connect("clicked", self.next_clicked)
        self.next_button.set_tooltip(_('Next'))
        self.toolbar.insert(self.next_button, -1)
        self.next_button.show()
       
        self.repeat_button = ToolButton('repeat')
        self.repeat_button.connect("clicked", self.repeat_clicked)
        self.repeat_button.set_tooltip(_('Repeat'))
        self.toolbar.insert(self.repeat_button, -1)
        self.repeat_button.show()
       
       	#We pack a vbox with our progressbar and time label, we pack our vbox in a ToolItem, 
        #and a ToolItem in our toolbar. Yay GUI's 
        self.progressbar_item = gtk.ToolItem()
        self.progressbar_item.set_expand(True)
        self.progressbar_vbox = gtk.VBox(homogeneous=False, spacing=0)
        self.progressbar_vbox.show()

        #Progress Bar
        self.adjustment = gtk.Adjustment(value=0, lower=0, upper=0, step_incr=0, page_incr=0, page_size=0)
        self.progressBar = gtk.HScale(self.adjustment)
    	self.progressBar.set_digits(int(0))
        self.progressBar.set_update_policy(gtk.UPDATE_CONTINUOUS)
        self.progressBar.set_value_pos(gtk.POS_TOP)
        self.progressBar.set_draw_value(False)
        self.progressBar.connect("button_release_event", self.buttonup)
        self.progressBar.connect("button_press_event", self.buttondown)
        self.progressBar.connect("change-value", self.barseek)
        self.progressBar.show()
        #Label Under Progress Bar
        self.toolbar_label = gtk.Label()
        self.toolbar_label.set_markup("<span foreground=\"white\" size=\"x-large\">00:00:00/00:00:00</span>")
        self.toolbar_label.show()
        #Packing Progress Bar and Label
        self.progressbar_vbox.pack_start(self.progressBar, expand=True, fill=True, padding=0)
        self.progressbar_vbox.pack_start(self.toolbar_label, expand=True, fill=True, padding=0)
	
        self.progressbar_item.add(self.progressbar_vbox)
        self.toolbar.insert(self.progressbar_item, -1)
        self.progressbar_item.show()

        self.open_button = ToolButton('stock-open')
        self.open_button.connect("clicked", self.openfiledialog)
        self.open_button.set_tooltip(_('Open File'))
        self.toolbar.insert(self.open_button, -1)
    	self.open_button.show()

        self.fullscreen_button = ToolButton('view-fullscreen')
        self.fullscreen_button.connect("clicked", self.fullscreen_clicked)
        self.fullscreen_button.set_tooltip(_('Enter Fullscreen'))
        self.toolbar.insert(self.fullscreen_button, -1)
        self.fullscreen_button.show()

        #Zoom buttons.
        self.fit_to_screen = ToolButton('zoom-native')
        self.fit_to_screen.connect("clicked", self.fit_screen_clicked)
        self.fit_to_screen.set_tooltip(_('Native Size'))
        self.toolbar.insert(self.fit_to_screen, -1)
        self.fit_to_screen.show()

        toolbox.add_toolbar('Media Player', self.toolbar)
        self.toolbar.show()
        self.set_toolbox(toolbox)
        toolbox.show()

        self.subwin_vbox = gtk.VBox(homogeneous=False, spacing=0)

        self.subwin = gtk.DrawingArea()
        color = gtk.gdk.color_parse('#000000')
        self.subwin.modify_bg(gtk.STATE_NORMAL, color)
        self.subwin.set_size_request(1200, 764)
        self.subwin.show()

        #I want the label backgrond color to be black, but I can't directly set that
        #because lables aren't drawable. So this just creates a label and adds it to
        #an event box.
        self.label = gtk.Label()
        self.label.show()
        self.event_box = gtk.EventBox()
        self.event_box.show()
        self.event_box.add(self.label)
        self.event_box.modify_bg(gtk.STATE_NORMAL, color)

        self.subwin_vbox.pack_start(self.subwin, expand=True, fill=True, padding=0)
        self.subwin_vbox.pack_start(self.event_box, expand=True, fill=True, padding=0)
        self.subwin_vbox.show()

        self.set_canvas(self.subwin_vbox)

        if handle.uri:
            self.file = handle.uri

        handle_uri_timer = gobject.timeout_add(10, self.handle_uri_timer)

hxplay.init()
